#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>

#include "fonctions.h"
#include "entrees_sorties.h"

#define TAILLE_V 10
#define NOMTUBESERVEUR "./serveur-rpc"

char nomTubeClient[80];
int fdW, fdR;


double call_somme(double a, double b) {
  printf("Appel a distance de somme(%lf, %lf)\n", a, b);
  write_int(fdW, SOMME);
  write_str(fdW, nomTubeClient, 80);
  write_double(fdW, a);
  write_double(fdW, b);

  double res = read_double(fdR);

  return res;
}

double call_mul(double a, double b) {
  printf("Appel a distance de mul(%lf, %lf)\n", a, b);
  write_int(fdW, MUL);
  write_str(fdW, nomTubeClient, 80);
  write_double(fdW, a);
  write_double(fdW, b);

  double res = read_double(fdR);

  return res;
}


/* res = a + b
 * a, b et res sont des vecteurs de meme taille
 */
void call_sommev(struct vecteur *a, struct vecteur *b, struct vecteur *res) {
  printf("Appel a distance de sommev\n");

  write_int(fdW, SOMMEV);
  write_str(fdW, nomTubeClient, 80);

  write_vecteur(fdW, a);
  write_vecteur(fdW, b);

  read_vecteur_allocated(fdR, res);
}

/* calcule a.b
 * a et b sont des vecteurs de meme taille
 */
double call_mulv(struct vecteur *a, struct vecteur *b) {
  printf("Appel a distance de mulv\n");

  write_int(fdW, MULV);
  write_str(fdW, nomTubeClient, 80);

  write_vecteur(fdW, a);
  write_vecteur(fdW, b);

  return read_double(fdR);
}



void tester() {
  double a = 14.5;
  double b = 18.5;

  /* test de la fonction somme */
  double res_somme = call_somme(a, b); /* calcul distant */
  double verif_somme = somme(a, b); /* calcul local */

  if(verif_somme != res_somme ) {
    fprintf(stderr, "Erreur dans la fonction somme\n");
    fprintf(stderr, "\tle calcul local donne %lf\n", verif_somme);
    fprintf(stderr, "\tle calcul recu donne %lf\n", res_somme);
    exit(EXIT_FAILURE);
  } else {
    printf("test de somme: OK\n");
  }


  /* test de la fonction mul */
  double res_mul = call_mul(a, b); /* calcul distant */
  double verif_mul = mul(a, b);	   /* calcul local */

  if(verif_mul != res_mul ) {
    fprintf(stderr, "Erreur dans la fonction mul\n");
    fprintf(stderr, "\tle calcul local donne %lf\n", verif_mul);
    fprintf(stderr, "\tle calcul recu donne %lf\n", res_mul);
    exit(EXIT_FAILURE);
  } else {
    printf("test de mul: OK\n");
  }



  struct vecteur *v1 = new_vecteur(TAILLE_V);
  struct vecteur *v2 = new_vecteur(TAILLE_V);

  int i;
  for(i=0; i<TAILLE_V; i++) {
    v1->values[i]=i%5;
    v2->values[i]=i%3;
  }

  /* test de la fonction sommev */
  struct vecteur* res_sommev = new_vecteur(TAILLE_V);
  struct vecteur* verif_sommev = new_vecteur(TAILLE_V);

  call_sommev(v1, v2, res_sommev); /* calcul distant */
  sommev(v1, v2, verif_sommev);	   /* calcul local */


  if(! vecteur_egal(verif_sommev, res_sommev) ) {
    fprintf(stderr, "Erreur dans la fonction sommev\n");
    fprintf(stderr, "\tle calcul local donne"); print_v(verif_sommev);
    fprintf(stderr, "\tle calcul recu donne"); print_v(res_sommev);
    exit(EXIT_FAILURE);
  } else {
    printf("test de sommev: OK\n");
  }


  /* test de la fonction mulv  */
  double res_mulv = call_mulv(v1, v2); /* calcul distant */
  double verif_mulv = mulv(v1, v2);    /* calcul local */

  if(verif_mulv != res_mulv ) {
    fprintf(stderr, "Erreur dans la fonction mulv\n");
    fprintf(stderr, "\tle calcul local donne %lf\n", verif_mulv);
    fprintf(stderr, "\tle calcul recu donne %lf\n", res_mulv);
    exit( EXIT_FAILURE );
  } else {
    printf("test de mul: OK\n");
  }
}

int main(int argc, char**argv) {

  /* Préparation de nomTubeClient (tube qui permettra les communications */
  /* serveur->client)                                                     */


  sprintf(nomTubeClient, "/tmp/client-rpc.%d", getpid());

  /* Création du tube nommé pour les communications serveur -> client */

  /* A COMPLETER */

  /* Ouverture de ce tube */

  /* A COMPLETER */



  /* ouverture du tube pour les communications client -> serveur */

  /* A COMPLETER */


  tester();


  /* on ferme les connexions */

  return EXIT_SUCCESS;

}
